// OSIKCS.H
// Converts data into Kansas City Standard audio file for use with OSI and
// Converts Kansas City Standard WAV Audio data to file
//
// Encoder based on Ed's Kansas City Standard and CUTS tape decoder/encoder dxforth@netbay.com.au
// Decoder mess rolled on my own
// 
// Mark Spankus 01/2020
// Released into public domain
//
#ifndef __OSIKCS_H
#define __OSIKCS_H

#include <stdlib.h>
#include <deque>

//RIFF files are stored little endian, not a problem for x86, x64
#if defined (__BIG_ENDIAN__)

#if !defined(swab)
void swab(char *from, char *to, int size)
{
	char a, b;
	while ( size ) {
		a = from[0];
		b = from[1];
		to[0] = b;
		to[1] = a;
		from += 2;
		to += 2;
		size -= 2;
	}
}
#endif

#define SWAP_UINT16(x) (swab( (char*)&x,(char*)&x,2))
#define SWAP_UINT32(x) (swab( (char*)&x,(char*)&x,4))

//#define SWAP_UINT16(x) (((x) >> 8) | ((x) << 8) )
//#define SWAP_UINT32(x) (((x) >> 24) | (((x) & 0x00FF0000) >> 8) | (((x) & 0x0000FF00) << 8) )

#else
#define SWAP_UINT16(x)	(x)
#define SWAP_UINT32(x)	(x)

#endif


#include <stdarg.h>
#include <string.h>
#if defined (_WIN32)
#include <tchar.h>
#else
#define _tmain      main
#define _tWinMain   WinMain
#define __targv     __argv
#define _T(x)      x
#define TCHAR	char
#define _TCHAR  char
#define _ttoi       atoi
#define _stprintf   sprintf
#define _vtprintf   vprintf
#define _tcscat     strcat
#define _tcscpy     strcpy
#define _tcslen     strlen
#define _tcsncpy    strncpy
#define _ftprintf   fprintf
#define _tfopen     fopen
#define _fputts     fputs
#define _vsntprintf vsnprintf
#define _tcsicmp    strcasecmp
#define _tcsnicmp	strncasecmp
#define _fstat      fstat
#define _fileno     fileno
#define _stat       stat
#define _ttoi		atoi

#endif

extern bool bVerbose;
void LogText(const TCHAR * szFormat, ...);
void LogData(unsigned char c);
enum eParity
{
	eNone = 0,
	eEven = 1,
	eOdd = 2,
	eMark = 3,
	eSpace = 4

};

	//converts sample rise & fall times into data bytes
class Emitter
{

public:
	Emitter(FILE* target);
	~Emitter();

	int add(int nCycles, size_t nPos);
	bool setTiming(int samples_per_second = 22050, int nBps = 300);
	void setFrame(int nBits = 8, int nStop = 2, eParity parity = eNone);

	int GetFEcount() { return FEcount; };
	int GetPEcount() { return PEcount; };
	double GetLeaderFreq() { return m_freq2400; };
	int GetBPS() { return m_nBps; };
	int GetWriteBytes() { return m_nWrote; };

		
protected:
	bool emit(bool bBit);
	
	std::deque<int> vQueue;		//place to store incoming wave times
	std::deque<int> :: const_iterator vQitr;
private:

	int m_nFreq;				//sample frequency of incoming WAV

	int m_nBps;					//expected BPS (300 or 600)
	int m_nBits;				//expected bitlength (7 or 8)
	int m_nStop;			    //expected # stop bits (1 or 2)
	eParity m_parity;			//expected parity (even, odd, none, mark, space)
	FILE* m_target;				//output file
	int m_nWrote;				//output count

	int m_nFrameSize;			//calculated # of samples for 1 bit
	int nJitter;				//tolerance of framesize

	int m_nframeLen;			//length of the expected serial word 8N2=11 
	unsigned char frame[12];    //serial word storage
	int m_nBitsIn;				//# bits since last byte
	int m_nStopBit1Pos;			//position of stop bit #1 in serial word (10)
	int m_nStopBit2Pos;			//position of stop bit #2 (or #1 again) in serial word (11)
	int m_nParityPos;			//position of parity bit in serial word if parity used


	bool bStart;
	bool bFrameError;
	int m_nCalcLeader;			//flag used when sampling input for actual freq & bps
	int FEcount;				//Frame Error count
	int PEcount;				//Parity Error count
	int nCount;
	size_t m_nCurPos;
	size_t m_nLastPos;
	size_t m_nStart;
	int m_Avg1200;				//~number of samples for 1/2 1200Hz 
	int m_Avg2400;				//~number of samples for 1/2 2400Hz
	double m_freq2400;			//sampled 2400Hz leader tone actual frequency


	int QSum();	


	

};
#endif
